{% extends "admin/change_form.html" %}
{% load i18n admin_urls admin_static admin_modify %}

{% block content %}
<div id="content-main">
  <div id="react-app"></div>
</div>

<script>
window.happymailerConfig = {{ happymailer_config|safe }};
</script>
<script type="text/javascript" src="{% static 'happymailer/client.js' %}"></script>
{% endblock %}
