{% extends "admin/happymailer/templatemodel/base.html" %}
{% load i18n %}
{% load admin_urls %}

{% block breadcrumbs_last %}
{% trans "Import" %}
{% endblock %}

{% block content %}
<h1>{% trans "Import templates" %}</h1>

<form action="" method="post" id="{{ opts.module_name }}_form" enctype="multipart/form-data">
  {% csrf_token %}

  <fieldset class="module_aligned">
    {% for field in form %}
    <div class="form-row">
      {{ field.errors }}
      {{ field.label_tag }}
      {{ field }}

      {% if field.field.help_text %}
      <p class="help">{{ field.field.help_text|safe }}</p>
      {% endif %}
    </div>
    {% endfor %}
  </fieldset>

  <div class="submit-row">
    <input type="submit" class="default" value="{% trans "Submit" %}">
  </div>
</form>

{% if result %}
<pre>{{ result }}</pre>
{% endif %}

{% endblock %}