# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-11 15:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('happymailer', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicaltemplate',
            name='layout',
            field=models.SlugField(null=True),
        ),
        migrations.AddField(
            model_name='historicaltemplate',
            name='subject',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='templatemodel',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='templatemodel',
            name='has_errors',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='historicaltemplate',
            name='archived_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='historicaltemplate',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='history', to='happymailer.TemplateModel'),
        ),
    ]
