{% extends "admin/change_list.html" %}

{% block object-tools %}
<ul class="object-tools">
  <li><a href="export/" class="addlink">Export</a></li>
  <li><a href="import/" class="addlink">Import</a></li>
</ul>
{% endblock %}